#include "hbresult_index.hh"
#include "hbresult_description.hh"
#include "hbresult_void.hh"
#include "headers_index.hh"
#include "headers_title.hh"
#include "headers_list.hh"
#include "view.img"
#include "xhb2html.hh"

Xhb2Html::Xhb2Html()
{
}

int	Xhb2Html::createDir(const char *lang, vector<StyxDomNode*> nodes,
			    const char* path)
{
  string				outputPath;
  int					fd;
  vector<StyxDomNode*>::iterator	it;
  FILE					*file;

  if (strlen(path) == 0)
    outputPath = "hbresult-html";
  else
    outputPath = outputPath + path + SLASH + "hbresult-html";
  if ((fd = open(outputPath.c_str(), O_RDONLY)) == -1)
    if (mkdir(outputPath.c_str(), 0755) == -1)
      return (0);
  close(fd);

  chdir(outputPath.c_str());

  fd = open("index.html", O_WRONLY | O_CREAT | O_TRUNC, 0644);
  write(fd, hbresult_index, strlen(hbresult_index));
  close(fd);

  fd = open("description.html", O_WRONLY | O_CREAT | O_TRUNC, 0644);
  if (!strcmp(lang, LANG_FR))
    write(fd, hbresult_description[1], strlen(hbresult_description[1]));
  else
    write(fd, hbresult_description[0], strlen(hbresult_description[0]));
  close(fd);

  fd = open("void.html", O_WRONLY | O_CREAT | O_TRUNC, 0644);
  write(fd, hbresult_void, strlen(hbresult_void));
  close(fd);

  fd = open("view.gif", O_WRONLY | O_CREAT | O_TRUNC, 0644);
  write(fd, view_img, view_size);
  close(fd);

  mkdir("headers", 0755);
  fd = open("headers/index.html", O_WRONLY | O_CREAT | O_TRUNC, 0644);
  write(fd, headers_index, strlen(headers_index));
  close(fd);

  fd = open("headers/void.html", O_WRONLY | O_CREAT | O_TRUNC, 0644);
  write(fd, hbresult_void, strlen(hbresult_void));
  close(fd);

  fd = open("headers/title.html", O_WRONLY | O_CREAT | O_TRUNC, 0644);
  write(fd, headers_title, strlen(headers_title));
  close(fd);

  file = fopen("headers/list.html", "w");
  fputs(headers_list_begin, file);
  for (it = nodes.begin(); it != nodes.end(); it++)
    {
      string	name = (*(((*it)->getAttributes("id")).begin()))->getValue();
      const char	*str = name.c_str();
      fprintf(file, headers_list_file, str, str, str);
    }
  fputs(headers_list_end, file);
  fclose(file);
  return (1);
}

int	Xhb2Html::go(StyxDom* dom, const char *outputPath, const char* lang)
{
  string				path;
  vector<StyxDomNode*>			headerNodes;
  vector<StyxDomNode*>::iterator	it;
  vector<StyxDomNode*>			attrNodes;
  string				name;

  // path = "/hb:headerbrowser/hb:headerfile"
  path = path + SLASH + XHB_TOPLEVEL_TAG + SLASH + XHB_HEADERFILE_TAG;
  headerNodes = dom->getFromXpath(path);
  if (headerNodes.size() && !createDir(lang, headerNodes, outputPath))
    return (0);
  for (it = headerNodes.begin(); it != headerNodes.end(); it++)
    {
      attrNodes = (*it)->getAttributes(XHB_ID_ATTR);
      name = (*(attrNodes.begin()))->getValue();
      if (!createHeaderDir(lang, name, dom, *it))
	return (0);
      if (!createGlobals(lang, name, dom, *it))
	return (0);
      if (!createFunctions(lang, name, dom, *it))
	return (0);
      if (!createTypedefs(lang, name, dom, *it))
	return (0);
      if (!createStructs(lang, name, dom, *it))
	return (0);
      if (!createUnions(lang, name, dom, *it))
	return (0);
      if (!createEnums(lang, name, dom, *it))
	return (0);
      if (!createEnumConstants(lang, name, dom, *it))
	return (0);
      if (!createMacroConstants(lang, name, dom, *it))
	return (0);
      if (!createMacroFunctions(lang, name, dom, *it))
	return (0);
      if (!createObjects(lang, name, dom, *it))
	return (0);
      chdir("..");
    }
  return (1);
}
